@@ -2,7 +2,13 @@
 # Date :  October 2017
 # Version 0.9
 # License GPL v3
-	
+
+setMethod("ext", signature(x="SpatExtent"), 
+	function(x, ...){ 
+		x
+	}
+)
+
 
 setMethod("ext", signature(x="missing"), 
 	function(x, ...){ 
@@ -10,23 +16,23 @@ setMethod("ext", signature(x="missing"),
 		e@ptr <- SpatExtent$new()
 		return(e)
 	}
-)	
-	
+)
+
 setMethod("ext", signature(x="numeric"), 
 	function(x, ...){ 
 		dots <- unlist(list(...))
 		x <- c(x, dots)
 		if (length(x) < 4) {
-			stop("insufficient number of elements (should be 4)")
+			error("ext", "insufficient number of elements (should be 4)")
 		}
 		if (length(x) > 4) {
-			warning("more elements than expected (should be 4)")
+			warn("ext", "more elements than expected (should be 4)")
 		}
 		names(x) <- NULL
 		e <- methods::new("SpatExtent")
 		e@ptr <- SpatExtent$new(x[1], x[2], x[3], x[4])
 		if (methods::validObject(e)) return(e)
-	}	
+	}
 )
 
 
@@ -36,22 +42,22 @@ setMethod("ext", signature(x="SpatRaster"),
 		e@ptr <- x@ptr$extent
 		return(e)
 	}
-)	
+)
 
 
-setMethod("ext", signature(x="SpatDataSet"), 
+setMethod("ext", signature(x="SpatRasterDataset"), 
 	function(x, ...){ 
 		e <- methods::new("SpatExtent")
 		e@ptr <- x[1]@ptr$extent
 		return(e)
 	}
-)	
+)
 
 
 setMethod("ext<-", signature("SpatRaster", "SpatExtent"), 
 	function(x, value) {
 		x@ptr$extent <- value@ptr
-		show_messages(x)
+		messages(x)
 	}
 )
 
@@ -61,10 +67,10 @@ setMethod("ext<-", signature("SpatRaster", "numeric"),
 		stopifnot(length(value) == 4)
 		e <- ext(value[1], value[2], value[3], value[4])
 		if (!e@ptr$valid) {
-			stop("not a valid extent specification")
+			error("ext<-", "not a valid extent specification")
 		}
 		x@ptr$extent <- e@ptr
-		show_messages(x)
+		messages(x)
 	}
 )
 
@@ -76,26 +82,26 @@ setMethod("ext", signature(x="SpatVector"),
 		e@ptr <- x@ptr$extent()
 		return(e)
 	}
-)	
+)
 
 
 setMethod("ext", signature(x="Extent"), 
 	function(x, ...) {
 		ext(as.vector(x))
 	}
-)	
+)
 
 setMethod("ext", signature(x="Raster"), 
-	function(x, ...) { 	
+	function(x, ...) { 
 		ext(x@extent)
 	}
-)	
+)
 
 setMethod("ext", signature(x="Spatial"), 
-	function(x, ...) { 	
+	function(x, ...) { 
 		ext(as.vector(t(x@bbox)))
 	}
-)	
+)
 
 
 
@@ -103,22 +109,22 @@ setMethod("xmin", signature(x="SpatExtent"),
 	function(x){ 
 		x@ptr$vector[1]
 	}
-)	
+)
 setMethod("xmax", signature(x="SpatExtent"), 
 	function(x){ 
 		x@ptr$vector[2]
 	}
-)	
+)
 setMethod("ymin", signature(x="SpatExtent"), 
 	function(x){ 
 		x@ptr$vector[3]
 	}
-)	
+)
 setMethod("ymax", signature(x="SpatExtent"), 
 	function(x){ 
 		x@ptr$vector[4]
 	}
-)	
+)
 
 
 setMethod("xmin<-", signature("SpatExtent", "numeric"), 
@@ -127,87 +133,91 @@ setMethod("xmin<-", signature("SpatExtent", "numeric"),
 		v[1] <- value
 		ext(v)
 	}
-)	
+)
 setMethod("xmax<-", signature("SpatExtent", "numeric"), 
 	function(x, ..., value){ 
 		v <- as.vector(x)
 		v[2] <- value
 		ext(v)
 	}
-)	
+)
 setMethod("ymin<-", signature("SpatExtent", "numeric"), 
 	function(x, ..., value){ 
 		v <- as.vector(x)
 		v[3] <- value
 		ext(v)
 	}
-)	
+)
 setMethod("ymax<-", signature("SpatExtent", "numeric"), 
 	function(x, ..., value){ 
 		v <- as.vector(x)
 		v[4] <- value
 		ext(v)
 	}
-)	
+)
 
 
 setMethod("xmin", signature(x="SpatRaster"), 
 	function(x){ 
 		xmin(ext(x))
 	}
-)	
+)
 setMethod("xmax", signature(x="SpatRaster"), 
 	function(x){ 
 		xmax(ext(x))
 	}
-)	
+)
 setMethod("ymin", signature(x="SpatRaster"), 
 	function(x){ 
 		ymin(ext(x))
 	}
-)	
+)
 setMethod("ymax", signature(x="SpatRaster"), 
 	function(x){ 
 		ymax(ext(x))
 	}
-)	
+)
 
 
 setMethod("xmin<-", signature("SpatRaster", "numeric"), 
 	function(x, ..., value){ 
 		v <- as.vector(ext(x))
 		v[1] <- value
+		x@ptr <- x@ptr$deepcopy()
 		ext(x) <- ext(v)
 		x
 	}
-)	
+)
 
 
 setMethod("xmax<-", signature("SpatRaster", "numeric"), 
 	function(x, ..., value){ 
 		v <- as.vector(ext(x))
 		v[2] <- value
+		x@ptr <- x@ptr$deepcopy()
 		ext(x) <- ext(v)
 		x
 	}
-)	
+)
 setMethod("ymin<-", signature("SpatRaster", "numeric"), 
 	function(x, ..., value){ 
 		v <- as.vector(ext(x))
 		v[3] <- value
+		x@ptr <- x@ptr$deepcopy()
 		ext(x) <- ext(v)
 		x
 	}
-)	
+)
 
 setMethod("ymax<-", signature("SpatRaster", "numeric"), 
 	function(x, ..., value){ 
 		v <- as.vector(ext(x))
 		v[4] <- value
+		x@ptr <- x@ptr$deepcopy()
 		ext(x) <- ext(v)
 		x
 	}
-)	
+)
 
 
 
@@ -215,22 +225,22 @@ setMethod("xmin", signature(x="SpatVector"),
 	function(x){ 
 		xmin(ext(x))
 	}
-)	
+)
 setMethod("xmax", signature(x="SpatVector"), 
 	function(x){ 
 		xmax(ext(x))
 	}
-)	
+)
 setMethod("ymin", signature(x="SpatVector"), 
 	function(x){ 
 		ymin(ext(x))
 	}
-)	
+)
 setMethod("ymax", signature(x="SpatVector"), 
 	function(x){ 
 		ymax(ext(x))
 	}
-)	
+)
 
 .ext2bb <- function(e) {
 	matrix(as.vector(e), ncol=2, byrow=TRUE)
@@ -240,10 +250,52 @@ setMethod("bbox", signature(obj="SpatRaster"),
 	function(obj){ 
 		.ext2bb(ext(obj))
 	}
-)	
+)
 
 setMethod("bbox", signature(obj="SpatVector"), 
 	function(obj){ 
 		.ext2bb(ext(obj))
 	}
-)	
+)
+
+
+
+setMethod("$", "SpatExtent",  
+	function(x, name) { 
+		as.vector(x)[name] 
+	}
+)
+
+
+setMethod("$<-", "SpatExtent",
+	function(x, name, value) { 
+		e <- as.vector(x)
+		e[name] <- value
+		ext(e)
+	}
+)
+
+
+
+setMethod("[", c("SpatExtent", "missing", "missing"),
+	function(x, i, j, ... , drop=FALSE) {
+		as.vector(x)
+	}
+)
+
+setMethod("[", c("SpatExtent", "numeric", "missing"),
+	function(x, i, j, ... , drop=FALSE) {
+		x <- as.vector(x)
+		x[i]
+	}
+)
+
+setReplaceMethod("[", c("SpatExtent", "numeric", "missing"),
+	function(x, i, j, value) {
+		e <- as.vector(x)
+		stopifnot(all(i %in% 1:4))
+		e[i] <- value
+		ext(e)
+	}
+)
+
