@@ -3,21 +3,21 @@
 # Version 1.0
 # License GPL v3
 
-	
+
 setMethod("init", signature(x="SpatRaster"), 
 	function(x, fun, filename="", overwrite=FALSE, wopt=list(), ...) {
-		opt <- .runOptions(filename, overwrite, wopt)
+		opt <- spatOptions(filename, overwrite, wopt)
 		if (is.character(fun)) {
 			fun <- fun[1]
 			if (fun %in% c("x", "y", "row", "col", "cell", "chess")) {
 				x@ptr <- x@ptr$initf(fun, TRUE, opt)
-				show_messages(x, "init")
+				messages(x, "init")
 			} else {
-				stop("unknown function")
+				error("init", "unknown function")
 			}
 		} else if (is.numeric(fun)) {
 			x@ptr <- x@ptr$initv(fun[1], opt)
-			show_messages(x, "init")
+			messages(x, "init")
 		} else {
 			out <- rast(x)
 			nc <- ncol(out)
@@ -28,8 +28,8 @@ setMethod("init", signature(x="SpatRaster"),
 				writeValues(out, r, b$row[i], b$nrows[i])
 			}
 			out <- writeStop(out)
-			return(out)		
+			return(out)
 		}
 	}
 )
-	
+
