@@ -3,27 +3,31 @@
 # Version 1.0
 # License GPL v3
 
-
+readAll <- function(x) {
+	ok <- x@ptr$readAll()
+	x <- messages(x)
+	invisible(ok)
+}
 
 setMethod("readStart", signature(x="SpatRaster"), 
 	function(x, ...) {
 		success <- x@ptr$readStart()
-		show_messages(x, "readStart")		
-		if (!success) stop("cannot open file for reading")
+		messages(x, "readStart")
+		if (!success) error("readStart,SpatRaster", "cannot open file for reading")
 		invisible(success)
 	}
 )
 
-setMethod("readStart", signature(x="SpatDataSet"), 
+setMethod("readStart", signature(x="SpatRasterDataset"), 
 	function(x, ...) {
 		success <- x@ptr$readStart()
-		show_messages(x, "readStart")		
-		if (!success) stop("cannot open file for reading")
+		messages(x, "readStart")
+		if (!success) error("readStart,SpatRasterDataset", "cannot open file for reading")
 		invisible(success)
 	}
 )
 
-#setMethod("readStart", signature(x="SpatDataSet"), 
+#setMethod("readStart", signature(x="SpatRasterDataset"), 
 #	function(x, ...) {
 #		nsd <- length(x)
 #		for (i in 1:nsd) {
@@ -31,7 +35,7 @@ setMethod("readStart", signature(x="SpatDataSet"),
 #			success <- readStart(y)
 #			x[i] <- y
 #		}
-#		show_messages(x, "readStart")		
+#		messages(x, "readStart")
 #		invisible(success)
 #	}
 #)
@@ -40,15 +44,15 @@ setMethod("readStart", signature(x="SpatDataSet"),
 setMethod("readStop", signature(x="SpatRaster"), 
 	function(x) {
 		success <- x@ptr$readStop()
-		show_messages(x, "readStop")		
+		messages(x, "readStop")
 		invisible(success)
 	}
 )
 
-setMethod("readStop", signature(x="SpatDataSet"), 
+setMethod("readStop", signature(x="SpatRasterDataset"), 
 	function(x) {
 		success <- x@ptr$readStop()
-		show_messages(x, "readStop")		
+		messages(x, "readStop")
 		invisible(success)
 	}
 )
@@ -56,12 +60,12 @@ setMethod("readStop", signature(x="SpatDataSet"),
 
 setMethod("writeStart", signature(x="SpatRaster", filename="character"), 
 	function(x, filename="", overwrite=FALSE, wopt=list(), ...) {
-		opt <- .runOptions(filename, overwrite, wopt)
+		opt <- spatOptions(filename, overwrite, wopt)
 		ok <- x@ptr$writeStart(opt)
-		show_messages(x, "writeStart")		
+		messages(x, "writeStart")
 		b <- x@ptr$getBlockSize(4, opt$memfrac)
 		b$row <- b$row + 1
-		b		
+		b
 	}
 )
 
@@ -69,11 +73,11 @@ setMethod("writeStart", signature(x="SpatRaster", filename="character"),
 setMethod("writeStop", signature(x="SpatRaster"), 
 	function(x) {
 		success <- x@ptr$writeStop()
-		show_messages(x, "writeStop")
+		messages(x, "writeStop")
 		f <- sources(x)$source
 		if (f != "") {
 			x <- rast(f)
-		}	
+		}
 		return(x)
 	} 
 )
@@ -86,7 +90,7 @@ setMethod("writeValues", signature(x="SpatRaster", v="vector"),
 		#	nrows <- length(v) / (ncol(x) * nlyr(x))
 		#}
 		success <- x@ptr$writeValues(v, start-1, nrows, 0, ncol(x))
-		show_messages(x, "writeValues")
+		messages(x, "writeValues")
 		invisible(success)
 	}
 )
@@ -94,9 +98,16 @@ setMethod("writeValues", signature(x="SpatRaster", v="vector"),
 
 setMethod("writeRaster", signature(x="SpatRaster", filename="character"), 
 function(x, filename="", overwrite=FALSE, wopt=list(), ...) {
-	opt <- .runOptions(filename, overwrite, wopt)
+	filename <- trimws(filename)
+	stopifnot(filename != "")
+	if (tools::file_ext(filename) %in% c("nc", "cdf") || isTRUE(wopt$filetype=="netCDF")) {
+		warn("writeRaster", "call writeCDF directly")
+		return ( writeCDF(x, filename=filename, overwrite=overwrite, ...) )
+	}
+	opt <- spatOptions(filename, overwrite, wopt)
 	x@ptr <- x@ptr$writeRaster(opt)
-	show_messages(x, "writeRaster")
+	x <- messages(x, "writeRaster")
+	invisible(rast(filename))
 }
 )
 
@@ -105,11 +116,11 @@ setMethod("writeVector", signature(x="SpatVector", filename="character"),
 function(x, filename, overwrite=FALSE, ...) {
 	filename <- trimws(filename)
 	if (filename == "") {
-		stop("provide a filename")
+		error("writeVector", "provide a filename")
 	}
 	lyrname <- gsub(".shp", "", basename(filename))
 	success <- x@ptr$write(filename, lyrname, "ESRI Shapefile", overwrite[1])
-	show_messages(x, "writeVector")
+	messages(x, "writeVector")
 	invisible(TRUE)
 }
 )
