@@ -5,10 +5,15 @@ setMethod("$<-", "SpatRaster",
 		if (inherits(value, "SpatRaster")) {
 			value <- value[[1]]
 			names(value) <- name
+		} else if (is.numeric(value)) {
+			y <- rast(x, nlyr=1)
+			values(y) <- value
+			value <- y
+			names(value) <- name
 		} else if (!is.null(value)) {
-			stop("the replacement value should be a SpatRaster")
+			error("$<-,SpatRaster", "the replacement value should be a SpatRaster or numeric")
 		}
-	
+
 		i <- which(name == names(x))[1]
 		if (is.na(i)) {
 			return(c(x, value))
@@ -16,7 +21,7 @@ setMethod("$<-", "SpatRaster",
 			if (i == 1) {
 				x <- c(value, x[[2:nlyr(x)]])
 			} else if (i == nlyr(x)) {
-				x <- c(x[[1:(nlyr(x)-1)]], value)			
+				x <- c(x[[1:(nlyr(x)-1)]], value)
 			} else {
 				x <- c(x[[1:(i-1)]], value, x[[(i+1):nlyr(x)]])
 			}
@@ -28,12 +33,22 @@ setMethod("$<-", "SpatRaster",
 
 setReplaceMethod("[[", c("SpatRaster", "character", "missing"),
 	function(x, i, j, value) {
-		if (nlyr(value) != length(i)) {
-			stop("length of names must be equal to the number of layers")
+		if (inherits(value, "SpatRaster")) {
+			if (nlyr(value) != length(i)) {
+				error(" [[,SpatRaster", "length of names must be equal to the number of layers")
+			}
+			names(value) <- i
+		} else if (length(i) > 1) {
+			if (NCOL(value) > 1) {
+				value <- as.list(data.frame(value))
+			} else {
+				stopifnot(length(i) == length(value))
+			}
+		} else if (!is.list(value)) {
+			value <- list(value)
 		}
-		names(value) <- i
 		for (k in 1:length(i)) {
-			eval(parse(text = paste("`$`(x, ", i[k], ") <- value[[k]]")))
+			eval(parse(text = paste0("x$", i[k], " <- value[[k]]")))
 		}
 		x
 	}
@@ -42,16 +57,16 @@ setReplaceMethod("[[", c("SpatRaster", "character", "missing"),
 setReplaceMethod("[[", c("SpatRaster", "numeric", "missing"),
 	function(x, i, j, value) {
 		if (nlyr(value) != length(i)) {
-			stop("length of indices must be equal to the number of layers")
+			error(" [[,SpatRaster,numeric", "length of indices must be equal to the number of layers")
 		}
 		if (any(i<1) | any(i > nlyr(x))) {
-			stop("indices must be between 1 and the number of layers")
+			error(" [[,SpatRaster,numeric", "indices must be between 1 and the number of layers")
 		}
 		for (k in 1:length(i)) {
 			if (i[k] == 1) {
 				x <- c(value[[k]], x[[2:nlyr(x)]])
 			} else if (i[k] == nlyr(x)) {
-				x <- c(x[[1:(nlyr(x)-1)]], value[[k]])			
+				x <- c(x[[1:(nlyr(x)-1)]], value[[k]])
 			} else {
 				x <- c(x[[1:(i[k]-1)]], value[[k]], x[[(i[k]+1):nlyr(x)]])
 			}
@@ -64,15 +79,15 @@ setReplaceMethod("[[", c("SpatRaster", "numeric", "missing"),
 
 setReplaceMethod("[", c("SpatRaster", "missing", "missing"),
 	function(x, i, j, value) {
-	
+
 		nl <- nlyr(x)
 		x <- rast(x)
-		
+
 		if (is.matrix(value)) {
 			if (all(dim(value) == c(ncell(x), nl))) {
 				e <- try( values(x) <- value)
 			} else {
-				stop("dimensions of the matrix do not match the SpatRaster")
+				error(" [,SpatRaster","dimensions of the matrix do not match the SpatRaster")
 			}
 		} else {
 			v <- try( matrix(nrow=ncell(x), ncol=nl) )
@@ -82,7 +97,7 @@ setReplaceMethod("[", c("SpatRaster", "missing", "missing"),
 			}
 		}
 		if (inherits(e, "try-error")) {
-			stop("cannot set values")
+			error(" [,SpatRaster", "cannot set values")
 		}
 		return(x)
 	}
@@ -97,7 +112,7 @@ setReplaceMethod("[", c("SpatRaster","numeric", "missing"),
 		if (narg > 0) { # row
 			i <- cellFromRowColCombine(x, i, 1:ncol(x))
 		}
-		
+
 		if (!is.null(dim(value))) {
 			#x@ptr <- x@ptr$replaceValues(i, value, ncol(value))
 			stopifnot(ncol(value) == nlyr(x))
@@ -121,7 +136,7 @@ setReplaceMethod("[", c("SpatRaster", "numeric", "numeric"),
 		x[i] <- value
 		x
 	}
-)	
+)
 
 
 setReplaceMethod("[", c("SpatRaster","missing", "numeric"),
@@ -140,7 +155,7 @@ setReplaceMethod("[", c("SpatRaster", "logical", "missing"),
 		x[i] <- value
 		x
 	}
-)	
+)
 
 
 setReplaceMethod("[", c("SpatRaster", "SpatRaster", "missing"),
@@ -148,16 +163,21 @@ setReplaceMethod("[", c("SpatRaster", "SpatRaster", "missing"),
 		theCall <- sys.call(-1)
 		narg <- length(theCall)-length(match.call(call=sys.call(-1)))
 		if (narg > 0) { # row
-			stop("you cannot use a SpatRaster as a row index")
+			error(" [,SpatRaster,SpatRaster", "you cannot use a SpatRaster as a row index")
 		}
 		if (inherits(value, "SpatRaster")) {
 			x <- mask(x, i, maskvalue=TRUE)
 			cover(x, value)
 		} else {
 			if (length(value) > 1) {
-				warning("the first replacement value is used for all cells")
+				v <- values(x)
+				v[as.logical(values(i))] <- value
+				values(x) <- v
+				x
+				#warn(" [,SpatRaster,SpatRaster", "the first replacement value is used for all cells")
+			} else {
+				mask(x, i, maskvalue=TRUE, updatevalue=value[1])
 			}
-			mask(x, i, maskvalue=TRUE, updatevalue=value[1])
 		}
 	}
 )
@@ -168,7 +188,7 @@ setReplaceMethod("[", c("SpatRaster", "SpatVector", "missing"),
 		theCall <- sys.call(-1)
 		narg <- length(theCall)-length(match.call(call=sys.call(-1)))
 		if (narg > 0) { # row
-			stop("you cannot use a SpatVector as a row index")
+			error(" [,SpatRaster,SpatVector", "you cannot use a SpatVector as a row index")
 		}
 		if (length(value) > 1) {
 			value <- rep_len(value, length.out=length(x))
