@@ -15,11 +15,11 @@ function(x, index, fun, ..., filename="", overwrite=FALSE, wopt=list()) {
 	txtfun <- .makeTextFun(fun)
 	if (inherits(txtfun, "character")) { 
 		if (txtfun %in% c("max", "min", "mean", "prod", "sum", "any", "all")) {
-			opt <- .runOptions(filename, overwrite, wopt)
+			opt <- spatOptions(filename, overwrite, wopt)
 			narm <- isTRUE(list(...)$na.rm)
-			x@ptr <- x@ptr$apply(index, txtfun, narm, nms, opt)	
-			return(show_messages(x, "tapp"))
-		}		
+			x@ptr <- x@ptr$apply(index, txtfun, narm, nms, opt)
+			return(messages(x, "tapp"))
+		}
 	}
 	fun <- match.fun(fun)
 
@@ -28,7 +28,7 @@ function(x, index, fun, ..., filename="", overwrite=FALSE, wopt=list()) {
 	out <- rast(x)
 	nlyr(out) <- length(uin)
 	names(out) <- nms
-	if (!readStart(x)) { stop(x@ptr$messages$getError()) }
+	readStart(x)
 	on.exit(readStop(x))
 	b <- writeStart(out, filename, overwrite, wopt)
 	for (i in 1:b$n) {
@@ -40,7 +40,7 @@ function(x, index, fun, ..., filename="", overwrite=FALSE, wopt=list()) {
 		v <- do.call(cbind, v)
 		writeValues(out, v, b$row[i], b$nrows[i])
 	}
-	out <- writeStop(out)	
+	out <- writeStop(out)
 	return(out)
 }
 )
