@@ -24,7 +24,7 @@ loadModule("spat", TRUE)
 	tv <- utils::packageVersion("terra")
 	#m <- paste("terra version", tv, "(beta-release)")
 	m <- paste("terra version", tv)
-	gdv <- gdal_version()
+	gdv <- gdal()
 	if (gdv < "3.0.0") {
 		a <- paste("\n\nNOTE: using GDAL version", gdv, "\nFor full functionality you need at least version 3.0.4\n")
 		m <- c(m, a)
