@@ -4,12 +4,47 @@
 
 //#include <memory> //std::addressof
 #include "gdal_priv.h"
-# include "gdal_info.h"
+#include "gdalio.h"
+#include "ogr_spatialref.h"
+
 
 #if GDAL_VERSION_MAJOR >= 3
 #include "proj.h"
 #endif
 
+// [[Rcpp::export(name = ".sameSRS")]]
+bool sameSRS(std::string x, std::string y) {
+	std::string msg;
+	SpatSRS srs;
+	if (!srs.set(x, msg)) return false;
+	return srs.is_same(y, false);
+}
+
+
+// [[Rcpp::export(name = ".getSRSname")]]
+std::string getCRSname(std::string s) {
+	OGRSpatialReference x;
+	OGRErr erro = x.SetFromUserInput(s.c_str());
+	if (erro != OGRERR_NONE) {
+		return "";
+	}
+	std::string node;
+	if (x.IsGeographic()) {
+		node = "geogcs";
+	} else {
+		node = "projcs";
+	}
+	return x.GetAttrValue(node.c_str());
+}
+
+// [[Rcpp::export(name = ".getLinearUnits")]]
+double getLinearUnits(std::string s) {
+	std::string msg;
+	SpatSRS srs;
+	if (!srs.set(s, msg)) return NAN;
+	return srs.to_meter();
+}
+
 
 
 // [[Rcpp::export(name = ".geotransform")]]
@@ -23,7 +58,7 @@ std::vector<double> geotransform(std::string fname) {
 		return out;
 	}
 
-	double gt[6];	
+	double gt[6];
 	if( poDataset->GetGeoTransform( gt ) != CE_None ) {
 		Rcpp::Rcout << "bad" << std::endl;
 	}
@@ -73,6 +108,32 @@ std::vector<std::vector<std::string>> sdsmetatdataparsed(std::string filename) {
 	return s;
 }
 
+// [[Rcpp::export(name = ".gdaldrivers")]]
+std::vector<std::vector<std::string>> gdal_drivers() {
+	size_t n = GetGDALDriverManager()->GetDriverCount();
+	std::vector<std::vector<std::string>> s(4);
+	s[0].reserve(n);
+	s[1].reserve(n);
+	s[2].reserve(n);
+	s[3].reserve(n);
+    GDALDriver *poDriver;
+    char **papszMetadata;
+	for (size_t i=0; i<n; i++) {
+	    poDriver = GetGDALDriverManager()->GetDriver(i);
+		s[0].push_back(poDriver->GetDescription());
+		s[3].push_back(poDriver->GetMetadataItem( GDAL_DMD_LONGNAME ) );
+
+		papszMetadata = poDriver->GetMetadata();
+		bool rst = CSLFetchBoolean( papszMetadata, GDAL_DCAP_RASTER, FALSE);
+		s[1].push_back(std::to_string(rst));
+		bool create = CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATE, FALSE);
+		bool copy = CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATECOPY, FALSE);
+		s[2].push_back(std::to_string(create + copy));
+	}
+	return s;
+}
+
+
 template <typename... Args>
 inline void warningNoCall(const char* fmt, Args&&... args ) {
     Rf_warningcall(R_NilValue, tfm::format(fmt, std::forward<Args>(args)... ).c_str());
@@ -101,7 +162,7 @@ static void __err_warning(CPLErr eErrClass, int err_no, const char *msg) {
             warningNoCall("%s (GDAL error class %d, #%d)", msg, eErrClass, err_no); 
             break; 
     }
-    return;	
+    return;
 }
 
 static void __err_error(CPLErr eErrClass, int err_no, const char *msg) {
@@ -109,7 +170,7 @@ static void __err_error(CPLErr eErrClass, int err_no, const char *msg) {
         case 0:
         case 1:
         case 2:
-            break; 		
+            break; 	
         case 3:
             warningNoCall("%s (GDAL error %d)", msg, err_no); 
             break;
@@ -151,20 +212,21 @@ void set_gdal_warnings(int level) {
 	if (level==4) {
 		CPLSetErrorHandler((CPLErrorHandler)__err_none);
 	} else if (level==1) {
-		CPLSetErrorHandler((CPLErrorHandler)__err_warning);			
+		CPLSetErrorHandler((CPLErrorHandler)__err_warning);		
 	} else if (level==2) {
-		CPLSetErrorHandler((CPLErrorHandler)__err_error);			
+		CPLSetErrorHandler((CPLErrorHandler)__err_error);		
 	} else {
 		CPLSetErrorHandler((CPLErrorHandler)__err_fatal);
-	} 		
+	} 	
 }
 
 
 // [[Rcpp::export(name = ".gdalinit")]]
 void gdal_init(std::string path) {
-	set_gdal_warnings(3);
+	set_gdal_warnings(2);
     GDALAllRegister();
     OGRRegisterAll(); 
+	CPLSetConfigOption("GDAL_MAX_BAND_COUNT", "9999999");
 	//GDALregistred = true;
 #if GDAL_VERSION_MAJOR >= 3
 	if (path != "") {
