@@ -24,29 +24,34 @@
 #endif
 
 
-enum SpatGeomType { points, lines, polygons, unknown };
+enum SpatGeomType { points, multipoints, lines, polygons, unknown };
+
 
 class SpatHole {
 	public:
 		std::vector<double> x, y;
 		SpatExtent extent;
+		//constructors
 		SpatHole();
 		SpatHole(std::vector<double> X, std::vector<double> Y);
+		//methods
 		size_t size() { return x.size(); }	
 };
 
 class SpatPart {
 	public:
 		std::vector<double> x, y; //, z;
-// should be here to support collections; add to constructors
-//		SpatGeomType gtype = unknown;
+		std::vector< SpatHole > holes; // polygons only
 		SpatExtent extent;
+
+		//constructors
 		SpatPart();
 		SpatPart(std::vector<double> X, std::vector<double> Y);
 		SpatPart(double X, double Y);
+
+		//methods
 		size_t size() { return x.size(); }
-		// for POLYGONS only
-		std::vector< SpatHole > holes;
+		//holes, polygons only
 		bool addHole(std::vector<double> X, std::vector<double> Y);
 		bool addHole(SpatHole h);
 		SpatHole getHole(unsigned i) { return( holes[i] ) ; }
@@ -60,9 +65,11 @@ class SpatGeom {
 		SpatGeomType gtype = unknown;
 		std::vector<SpatPart> parts;
 		SpatExtent extent;
+		//constructors
 		SpatGeom();
 		SpatGeom(SpatGeomType g);
 		SpatGeom(SpatPart p);
+		//methods
 		bool unite(SpatGeom g);
 		bool addPart(SpatPart p);
 		bool addHole(SpatHole h);
@@ -76,6 +83,8 @@ class SpatGeom {
 };
 
 
+class SpatVectorCollection;
+
 class SpatVector {
 
 	public:
@@ -92,7 +101,7 @@ class SpatVector {
 		SpatVector(std::vector<double> x, std::vector<double> y, SpatGeomType g, std::string crs);
 		SpatVector(std::vector<std::string> wkt);
 
-		SpatGeom window;
+		SpatGeom window; // for point patterns, must be polygon
 
 		std::vector<std::string> get_names();
 		void set_names(std::vector<std::string> s);
@@ -100,7 +109,10 @@ class SpatVector {
 		unsigned ncol();
 		unsigned nxy();
 
+		SpatVector deepCopy() {return *this;}
+
 		SpatExtent getExtent();
+		bool is_geographic();
 		bool is_lonlat();
 		bool could_be_lonlat();
 		std::string type();
@@ -109,7 +121,6 @@ class SpatVector {
 		//std::vector<std::string> getCRS();
 		//void setCRS(std::vector<std::string> _crs);
 
-
 		bool setSRS(std::string _srs) {
 			std::string msg;
 			if (!srs.set(_srs, msg)){
@@ -119,34 +130,20 @@ class SpatVector {
 			return true;	
 		}
 
-/*
-#ifdef useGDAL	
-		bool setSRS(OGRSpatialReference *poSRS, std::string &msg) {
-			if (!srs.set(poSRS, msg)){
-				addWarning("Cannot set SRS to vector: "+ msg);
-				return false;
-			}
-			return true;				
-		}
-#endif		
-*/
-
 		std::string getSRS(std::string x) {
 			return srs.get(x);
 		}
-		//std::string getPRJ();
-		//void setPRJ(std::string PRJ);
 
 		SpatGeom getGeom(unsigned i);
 		bool addGeom(SpatGeom p);
 		bool setGeom(SpatGeom p);
+		std::vector<std::vector<double>> getGeometry();
 		SpatDataFrame getGeometryDF();
 		std::vector<std::string> getGeometryWKT();
 
 		std::vector<std::vector<double>> coordinates();
 
 		SpatVector project(std::string crs);
-		//std::vector<std::vector<double>> test(std::vector<double> x, std::vector<double> y, std::string fromcrs, std::string tocrs);
 
 		SpatVector subset_cols(int i);
 		SpatVector subset_cols(std::vector<int> range);
@@ -157,17 +154,22 @@ class SpatVector {
 
 		std::vector<double> area();
 		std::vector<double> length();
-		SpatDataFrame distance(SpatVector x, bool pairwise);
-		SpatDataFrame distance();
+		std::vector<double> distance(SpatVector x, bool pairwise);
+		std::vector<double> distance(bool sequential);
+
+		std::vector<std::vector<size_t>> knearest(size_t k);
 
 		size_t size();
 		SpatVector as_lines();
-		SpatVector as_points();
+		SpatVector as_points(bool multi);
 		SpatVector remove_holes();
+		SpatVector get_holes();
+		SpatVector set_holes(SpatVector x, size_t i);
 
 		bool read(std::string fname);
 		
 		bool write(std::string filename, std::string lyrname, std::string driver, bool overwrite);
+		
 #ifdef useGDAL
 		GDALDataset* write_ogr(std::string filename, std::string lyrname, std::string driver, bool overwrite);
 		GDALDataset* GDAL_ds();
@@ -190,41 +192,123 @@ class SpatVector {
 			return df.add_column(x, name);
 		}
 
+		void remove_df() {
+			SpatDataFrame empty;
+			df = empty;
+		};
+
 		bool remove_column(std::string field) {
 			return df.remove_column(field);
 		};
 		bool remove_column(int i) {
 			return df.remove_column(i);
 		};
-
+		std::vector<std::string> get_datatypes() {
+			return df.get_datatypes();
+		}
 
 		SpatMessages msg;
 		void setError(std::string s) { msg.setError(s); }
 		void addWarning(std::string s) { msg.addWarning(s); }
 		bool hasError() { return msg.has_error; }
 		bool hasWarning() { return msg.has_warning; }
+		std::string getWarnings() { return msg.getWarnings();}
+		std::string getError() { return msg.getError();}
 
 		SpatVector point_buffer(double d, unsigned quadsegs);
-
-		std::vector<bool> is_valid();
-		SpatVector make_valid();
         SpatVector buffer(double d, unsigned segments, unsigned capstyle);
 
-		SpatVector aggregate(std::string field, bool dissolve);
+		SpatVector append(SpatVector x, bool ignorecrs);
 		SpatVector disaggregate();
 		SpatVector shift(double x, double y);
+		SpatVector rescale(double f, double x0, double y0);
+		SpatVector transpose();
+		SpatVector flip(bool vertical);	
+		SpatVector rotate(double angle, double x0, double y0);
 
+//ogr 
+		std::vector<bool> is_valid();
+		SpatVector make_valid();
 //geos
+		std::vector<bool> geos_isvalid();
+		std::vector<std::string> geos_isvalid_msg();
+
+		SpatVector allerretour();
+		SpatVectorCollection bienvenue();
+		SpatVector aggregate(bool dissolve);
+		SpatVector aggregate(std::string field, bool dissolve);
         SpatVector buffer2(double d, unsigned segments, unsigned capstyle);
+		SpatVector centroid();
+		SpatVector crop(SpatExtent e);
+		SpatVector crop(SpatVector e);
+		SpatVector voronoi(SpatVector e, double tolerance, int onlyEdges);		
+		SpatVector delauny(double tolerance, int onlyEdges);		
+		SpatVector convexhull();
 		SpatVector intersect(SpatVector v);
+		SpatVector unite(SpatVector v);
+		SpatVector erase(SpatVector v);
+		SpatVector cover(SpatVector v, bool identity);
+		SpatVector symdif(SpatVector v);
+		std::vector<int> relate(SpatVector v, std::string relation);
+		std::vector<int> relate(std::string relation, bool symmetrical);
+		std::vector<int> relateFirst(SpatVector v, std::string relation);
+		std::vector<double> geos_distance(SpatVector v, bool parallel);
+		std::vector<double> geos_distance(bool sequential);
+
+		SpatVector nearest_point(SpatVector v, bool parallel);
+		SpatVector nearest_point();
+		SpatVector sample(unsigned n, std::string method, bool by_geom, std::string strata, unsigned seed);
+
+		SpatVector unaryunion();
+
 };
 
 
 
 class SpatVectorCollection {
 
-	public:
+	private:
 		std::vector<SpatVector> v;
+
+	public:
+		SpatMessages msg;
+		void setError(std::string s) { msg.setError(s); }
+		void addWarning(std::string s) { msg.addWarning(s); }
+		bool hasError() { return msg.has_error; }
+		bool hasWarning() { return msg.has_warning; }
+		std::string getWarnings() { return msg.getWarnings();}
+		std::string getError() { return msg.getError();}
+
+		size_t size() { return v.size(); }
+		void push_back(SpatVector x) { v.push_back(x); };
+		bool replace(SpatVector x, size_t i) { 
+			if (i < size()) {
+				v[i] = x; 
+				return true;
+			} else {
+				return false;
+			}
+		}
+		SpatVectorCollection subset(std::vector<size_t> i) { 
+			SpatVectorCollection out;
+			for (size_t j=0; j<size(); j++) {
+				if (i[j] < size()) {
+					out.push_back(v[i[j]]); 
+				} 
+			}
+			return out;
+		}
+
+		SpatVector get(size_t i) { 
+			SpatVector out;
+			out.msg = msg;
+			if (i < size()) {
+				out = v[i];
+			} else {
+				out.setError("invalid index");
+			}
+			return out;
+		}
 		
 };
 
